local moduleName = 'Automation'
local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local module = idTweaks:newModule(moduleName, 'AceConsole-3.0', 'AceEvent-3.0')

module.defaults = {
  profile = {
    enabled = true,

    accepting = {
      acceptHearthstoneBind = true,
      acceptPVPRelease = true,
      acceptResurrection = true,
      acceptSummon = true,
    },

    vendors = {
      repairGear = true,
      sellGreyItems = true,
    },

    movement = {
      dismount = true,
    },
  }
}

module.options = {
  name = moduleName,
  type = 'group',
  args = {
    enabled = {
      order = 1,
      name = 'Enabled',
      desc = 'Enables / Disables the Module',
      type = 'toggle',
      get = function()
        return module.db.profile.enabled
      end,
      set = function(_, value)
        module.db.profile.enabled = value
        module:check()
      end,
    },

    accepting = {
      name = 'Accepting',
      desc = 'Automatically accept certain offers.',
      type = 'group',
      disabled = function()
        return not module.db.profile.enabled
      end,
      args = {
        acceptHearthstoneBind = {
          name = 'Accept Hearthstone Bind',
          desc = 'Automatically accepts hearthstone binds.',
          type = 'toggle',
          get = function()
            return module.db.profile.accepting.acceptHearthstoneBind
          end,
          set = function(_, value)
            module.db.profile.accepting.acceptHearthstoneBind = value
          end,
        },
        acceptPVPRelease = {
          name = 'Accept PVP Release',
          desc = 'Automatically accepts release on death in PVP.',
          type = 'toggle',
          get = function()
            return module.db.profile.accepting.acceptPVPRelease
          end,
          set = function(_, value)
            module.db.profile.accepting.acceptPVPRelease = value
          end,
        },
        acceptResurrection = {
          name = 'Accept Resurrection',
          desc = 'Automatically accepts resurrections.',
          type = 'toggle',
          get = function()
            return module.db.profile.accepting.acceptResurrection
          end,
          set = function(_, value)
            module.db.profile.accepting.acceptResurrection = value
          end,
        },
        acceptSummon = {
          name = 'Accept Summon',
          desc = 'Automatically accepts summons.',
          type = 'toggle',
          get = function()
            return module.db.profile.accepting.acceptSummon
          end,
          set = function(_, value)
            module.db.profile.accepting.acceptSummon = value
          end,
        },
      }
    },
    vendors = {
      name = 'Vendors',
      desc = 'Automate vendor interactions.',
      type = 'group',
      disabled = function()
        return not module.db.profile.enabled
      end,
      args = {
        repairGear = {
          name = 'Repair gear',
          desc = 'Automatically repairs your gear.',
          type = 'toggle',
          get = function()
            return module.db.profile.vendors.repairGear
          end,
          set = function(_, value)
            module.db.profile.vendors.repairGear = value
          end,
        },
        sellGreyItems = {
          name = 'Sell Grey Items',
          desc = 'Automatically sells trash at vendors.',
          type = 'toggle',
          get = function()
            return module.db.profile.vendors.sellGreyItems
          end,
          set = function(_, value)
            module.db.profile.vendors.sellGreyItems = value
          end,
        },
      }
    },
    movement = {
      name = 'Movement',
      desc = 'Automate movement.',
      type = 'group',
      disabled = function()
        return not module.db.profile.enabled
      end,
      args = {
        repairGear = {
          name = 'Dismount',
          desc = 'Automatically dismounts you when you perform an action that requires you to not be mounted..',
          type = 'toggle',
          get = function()
            return module.db.profile.movement.dismount
          end,
          set = function(_, value)
            module.db.profile.movement.dismount = value
          end,
        },
      }
    }
  },
}

function module:enable()
  module:RegisterEvent('CONFIRM_BINDER')
  module:RegisterEvent('CONFIRM_SUMMON')
  module:RegisterEvent('MERCHANT_SHOW')
  module:RegisterEvent('PLAYER_DEAD')
  module:RegisterEvent('RESURRECT_REQUEST')
  module:RegisterEvent('UI_ERROR_MESSAGE')
end

function module:disable()
  module:UnregisterEvent('CONFIRM_BINDER')
  module:UnregisterEvent('CONFIRM_SUMMON')
  module:UnregisterEvent('MERCHANT_SHOW')
  module:UnregisterEvent('PLAYER_DEAD')
  module:UnregisterEvent('RESURRECT_REQUEST')
  module:UnregisterEvent('UI_ERROR_MESSAGE')
end

function module:sellGreyItems()
  local link
  for bag = 0, 4 do
    for slot = 0, GetContainerNumSlots(bag) do
      link = GetContainerItemLink(bag, slot)
      if link and select(3, GetItemInfo(link)) == 0 then
        UseContainerItem(bag, slot)
      end
    end
  end
end

function module:repairGear()
  local cost = GetRepairAllCost()
  local funds = GetMoney()

  if (funds < cost) then
    module:Print('Repair: Insufficient Funds')
  end

  if (cost > 0) then
    RepairAllItems()
    module:Print('Repair: ' .. GetCoinTextureString(cost))
  end
end

function module:CONFIRM_BINDER()
  local db = module.db.profile

  if db.accepting.acceptHearthstoneBind == true then
    StaticPopup1Button1:Click()
  end
end

function module:CONFIRM_SUMMON()
  if not module.db.profile.accepting.acceptSummon then return end
  if UnitAffectingCombat('player') then return end

  StaticPopup1:Hide()
  ConfirmSummon()
end

function module:MERCHANT_SHOW()
  local db = module.db.profile

  if db.vendors.sellGreyItems == true then
    module:sellGreyItems()
  end

  if CanMerchantRepair() and db.vendors.repairGear == true then
    module:repairGear()
  end
end

function module:PLAYER_DEAD()
  if not module.db.profile.accepting.acceptPVPRelease then return end
  if UnitAffectingCombat('player') then return end
  if C_DeathInfo.GetSelfResurrectOptions() and #C_DeathInfo.GetSelfResurrectOptions() > 0 then return end

  local inInstance, instanceType = IsInInstance()
  if inInstance and instanceType == 'pvp' then
    RepopMe()
  end
end

function module:RESURRECT_REQUEST()
  if not module.db.profile.accepting.acceptResurrection then return end
  if GetCorpseRecoveryDelay() ~= 0 then return end

  StaticPopup1:Hide()
  AcceptResurrect()
end

local mountErrorMessages = {
  [ERR_ATTACK_MOUNTED] = true,
  [ERR_MOUNT_ALREADYMOUNTED] = true,
  [ERR_NOT_WHILE_MOUNTED] = true,
  [ERR_TAXIPLAYERALREADYMOUNTED] = true,
  [SPELL_FAILED_NOT_MOUNTED] = true,
}

function module:UI_ERROR_MESSAGE(_, _, errorMessage)
  if not module.db.profile.movement.dismount then return end
  if not IsMounted() then return end

  if (mountErrorMessages[errorMessage]) then
    Dismount()
  end
end
